"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useDebugConcurrent = void 0;
const react_1 = require("react");
/**
 * Used to debug React's concurrent features,
 * more specifically, it allows you to
 * attach callbacks to the lifecycle of
 * low priority and high priority renders.
 *
 * To make your results as accurate as possible,
 * make sure this hook is the first hook in your
 * component.
 */
const useDebugConcurrent = ({ onFirstRenderStart, onFirstRenderEnd, onLowPriorityStart, onLowPriorityEnd, onHighPriorityStart, onHighPriorityEnd, }) => {
    // We use a deferred value to determine
    // whether the current render is a low
    // priority render or not.
    // As the probe is created on the render
    // the object we create has always a new
    // reference, which means that the deferred
    // value will always be different from the
    // previous render.
    const probe = {};
    const deferredProbe = (0, react_1.useDeferredValue)(probe);
    const isFirstRenderRef = (0, react_1.useRef)(true);
    const isFirstRender = isFirstRenderRef.current;
    const isLowPriority = probe === deferredProbe;
    if (isFirstRender) {
        isFirstRenderRef.current = false;
        onFirstRenderStart === null || onFirstRenderStart === void 0 ? void 0 : onFirstRenderStart();
    }
    else {
        if (isLowPriority) {
            onLowPriorityStart === null || onLowPriorityStart === void 0 ? void 0 : onLowPriorityStart();
        }
        else {
            onHighPriorityStart === null || onHighPriorityStart === void 0 ? void 0 : onHighPriorityStart();
        }
    }
    useIsomorphicLayoutEffect(() => {
        // We consider not only the render
        // of the component itself, but the render
        // of the whole component sub-tree.
        // This way, when effects run, we know
        // that the whole component sub-tree
        // has been rendered.
        if (isFirstRender) {
            onFirstRenderEnd === null || onFirstRenderEnd === void 0 ? void 0 : onFirstRenderEnd();
        }
        else {
            if (isLowPriority) {
                onLowPriorityEnd === null || onLowPriorityEnd === void 0 ? void 0 : onLowPriorityEnd();
            }
            else {
                onHighPriorityEnd === null || onHighPriorityEnd === void 0 ? void 0 : onHighPriorityEnd();
            }
        }
    });
};
exports.useDebugConcurrent = useDebugConcurrent;
const isServer = typeof window === "undefined";
const useIsomorphicLayoutEffect = isServer ? react_1.useEffect : react_1.useLayoutEffect;
